﻿// --------------------------------------------------------------------------------------
//
// RobDuoCS2005                                                    Ulrich Müller 02.12.08
//                                                                 Aspire C# 2005 Express
// Zusammenspiel von zwei Industry Säulen-Robots 
//   mit der Klasse FishRobot, Basis ist die Assembly FishFace2005.DLL
//
// ROBO Interface mit M1 (I1,I2) Säule,        M2 (I3,I4) ArmHorizontal, 
// (ft2)              M2 (I5,I6) ArmVertikal,  M4 (I7,I8) Greifer
// Intelligent Interface mit gleicher Belegung (ft1)
//
// ft1 holt von Pos1 eine gelbe Tonne und stellt sie auf Pos2 ab
// ft2 holt von Pos2 eine gelbe Tonne und wirft sie von Pos3 in den Papierkorb
//     läuft in zweitem Thread
// Nachschub für Pos1 manuell aus dem Papierkorb
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using FishFace40;

namespace RobDuoCS2005 {
  public partial class frmMain : Form {

    // --- Positionskonstanten ---

    const int cSaule1Pos1 = 32;
    const int cArmH1Pos1  = 88;
    const int cArmV1Pos1  = 68;
    const int cArmV1Pos1o = 28;
    
    const int cSaule1Pos2 = 106;
    const int cArmH1Pos2  = 77;
    const int cArmV1Pos2  = 68;
    const int cArmV1Pos2o = 28;

    const int cSaule2Pos2 = 106;
    const int cArmH2Pos2  = 77;
    const int cArmV2Pos2  = 108;

    const int cSaule2Pos3 = 12;
    const int cArmH2Pos3  = 32;
    const int cArmV2Pos3  = 66;

    // --- Globale Variable ---

    private FishRobot      ft1;
    private AutoResetEvent rob1Ready;
    private FishRobot      ft2;
    private Thread         rob2;
    private AutoResetEvent rob2Ready;

    
    // --- Ereignis-Routine zur Positionsanzeige ft1 ----------
    private void Position1(object sender, int[] actPos) {
      string Pos = "Position Robot 1: ";
      for (int i = 0; i < actPos.Length - 1; i++) {
        Pos += actPos[i].ToString() + " - ";
      }
      lblPosition1.Text = Pos + actPos[actPos.Length - 1];
    }
    // --- ft1 : Holen Teil von Pos1 ---
    private void HolenPos1() {
      // --- Arm aufwärts ---
      ft1.MoveTo(ft1.MotCntl[0].actPos, ft1.MotCntl[1].actPos, cArmV1Pos1o);
      // --- Robot auf Position 1 ---
      ft1.MoveTo(cSaule1Pos1, cArmH1Pos1);
      // --- Arm abwärts --
      ft1.MoveTo(ft1.MotCntl[0].actPos, ft1.MotCntl[1].actPos, cArmV1Pos1);
      // --- Greifer schliessen ---
      ft1.MoveTo(ft1.MotCntl[0].actPos, ft1.MotCntl[1].actPos,
                 ft1.MotCntl[2].actPos, ft1.MotCntl[3].maxPos);
    }
    // --- ft1 : Bringen Teil nach Pos2 ---
    private void BringenPos2() {
      // --- Arm aufwärts ---
      ft1.MoveTo(ft1.MotCntl[0].actPos, ft1.MotCntl[1].actPos, cArmV1Pos2o);
      // --- Robot auf Position 2 ---
      ft1.MoveTo(cSaule1Pos2, cArmH1Pos2);
      // --- Arm abwärts ---
      ft1.MoveTo(ft1.MotCntl[0].actPos, ft1.MotCntl[1].actPos, cArmV1Pos2);
      // --- Greifer öffnen ---
      ft1.MoveTo(ft1.MotCntl[0].actPos, ft1.MotCntl[1].actPos, ft1.MotCntl[2].actPos, 0);
    }
    // --- Ereignis-Routine zur Positionsanzeige ft2 ---
    private void Position2(object sender, int[] actPos) {
      // --- Ereignis-Routine zur Positionsanzeige ----------
      string Pos = "Position Robot 2 : ";
      for (int i = 0; i < actPos.Length - 1; i++) {
        Pos += actPos[i].ToString() + " - ";
      }
      lblPosition2.Text = Pos + actPos[actPos.Length - 1];
    }
    // --- thread-Routine für ft2 ---
    private void Rob2Execute() {
      ft2.MoveHome();
      rob2Ready.Set();
      do {
        rob1Ready.WaitOne();
        HolenPos2();
        rob2Ready.Set();
        rob1Ready.WaitOne();
        BringenPos3();
        rob2Ready.Set();
      } while (!ft1.Finish() && !ft2.Finish());
    }
    // --- ft2 : Holen Teil von Pos2 --
    private void HolenPos2() {
      // --- Arm aufwärts ---
      ft2.MoveTo(ft2.MotCntl[0].actPos, ft2.MotCntl[1].actPos, 0);
      // --- Robot auf Position 2 ---
      ft2.MoveTo(cSaule2Pos2, cArmH2Pos2);
      // --- Arm abwärts ---
      ft2.MoveTo(ft2.MotCntl[0].actPos, ft2.MotCntl[1].actPos, cArmV2Pos2);
      // --- Greifer schließen ---
      ft2.MoveTo(ft2.MotCntl[0].actPos, ft2.MotCntl[1].actPos, 
                 ft2.MotCntl[2].actPos, ft2.MotCntl[3].maxPos);
    }
    // --- ft2 : Bringen Teil nach Pos3 ---
    private void BringenPos3() {
      // --- Arm aufwärts ---
      ft2.MoveTo(ft2.MotCntl[0].actPos, ft2.MotCntl[1].actPos, 0);
      // --- Robot auf Position 3 ---
      ft2.MoveTo(cSaule2Pos3, cArmH2Pos3);
      // --- Greifer öffnen -> in den Papierkorb ---
      ft2.MoveTo(ft2.MotCntl[0].actPos, ft2.MotCntl[1].actPos, ft2.MotCntl[2].actPos, 0);
   }
    
    // --- Steuerung ft1 und Start und Ende ---
    private void Action() {
      rob1Ready = new AutoResetEvent(false);
      rob2 = new Thread(new ThreadStart(Rob2Execute));
      rob2.IsBackground = true;
      rob2Ready = new AutoResetEvent(false);
      rob2.Start();
      lblStatus.Text = "Auf Home-Position";
      ft1.MoveHome();
      rob1Ready.Set();
      do {
        rob2Ready.WaitOne();
        lblStatus.Text = "Holen Teile";
        HolenPos1();
        rob1Ready.Set();
        rob2Ready.WaitOne();
        lblStatus.Text = "Bringen Teile";
        BringenPos2();
        rob1Ready.Set();
      } while (!ft1.Finish() && !ft2.Finish());
      rob2.Join();
    }

    #region --- Programm-Kontrolle ---
    public frmMain() {
      InitializeComponent();
      Label.CheckForIllegalCrossThreadCalls = false;
      ft1 = new FishRobot(new int[,] { { 1, 180 }, { 2, 112 }, { 3, 120 }, { 4, 22 } });
      ft2 = new FishRobot(new int[,] { { 1, 180 }, { 2, 112 }, { 3, 120 }, { 4, 26 } });
      ft2.PositionChange += new FishRobot.CommonDelegate(Position2);
      ft1.PositionChange += new FishRobot.CommonDelegate(Position1);
    }

    private void cmdAction_Click(object sender, EventArgs e) 
    {
      try {
        ft1.OpenInterface(IFTypen.ftIntelligent_IF, 1, 5);
        ft2.OpenInterface(IFTypen.ftROBO_first_USB, 0);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        

      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        ft1.CloseInterface();
        ft2.CloseInterface();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      if (cmdEnde.Text == "&HALT") {
        ft1.NotHalt = true; ft2.NotHalt = true;
      }
      else this.Close();
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}